@@ -1,9 +1,13 @@
-/*
- * Initialize a kTLS socket. In some cases initialization might
- * be handled by the TLS library.
+/**
+ * @file ktls.c
+ * @brief Initialize a kTLS socket. In some cases initialization might
+ *	  be handled by the TLS library
  *
+ * @copyright
  * Copyright (c) 2022 Oracle and/or its affiliates.
- *
+ */
+
+/*
  * ktls-utils is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; version 2.
@@ -19,7 +23,7 @@
  * 02110-1301, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -42,6 +46,13 @@
 #include "tlshd.h"
 #include "netlink.h"
 
+/**
+ * @brief Concatenate two NUL-terminated C strings
+ * @param[in]     str1  left-hand string
+ * @param[in]     str2  right-hand string
+ *
+ * @returns "str1" followed by "str2"
+ */
 static char *tlshd_string_concat(char *str1, const char *str2)
 {
 	size_t len = 0;
@@ -69,6 +80,14 @@ static char *tlshd_string_concat(char *str1, const char *str2)
 }
 
 #ifdef HAVE_GNUTLS_TRANSPORT_IS_KTLS_ENABLED
+/**
+ * @brief Determine if a session is kTLS-enabled
+ * @param[in]     session  An established TLS session
+ * @param[in]     read     Which side of the duplex to check
+ *
+ * @retval  true   The session is prepared to use kTLS
+ * @retval  false  The session is not prepared to use kTLS
+ */
 static bool tlshd_is_ktls_enabled(gnutls_session_t session, unsigned read)
 {
 	int ret;
@@ -90,6 +109,14 @@ static bool tlshd_is_ktls_enabled(gnutls_session_t session, unsigned read)
 }
 
 #else
+/**
+ * @brief Determine if a session is kTLS-enabled
+ * @param[in]     session  An established TLS session
+ * @param[in]     read     Which side of the duplex to check
+ *
+ * @retval  true   The session is prepared to use kTLS
+ * @retval  false  The session is not prepared to use kTLS
+ */
 static bool tlshd_is_ktls_enabled(__attribute__ ((unused)) gnutls_session_t session,
 				  __attribute__ ((unused)) unsigned read)
 {
@@ -97,6 +124,16 @@ static bool tlshd_is_ktls_enabled(__attribute__ ((unused)) gnutls_session_t sess
 }
 #endif
 
+/**
+ * @brief Call setsockopt(3), with error logging
+ * @param[in]     sock     An open socket descriptor
+ * @param[in]     read     Read or write side
+ * @param[in]     info     The value to set
+ * @param[in]     infolen  The size of "info", in bytes
+ *
+ * @retval  true   The option was set successfully
+ * @retval  false  The option was not set
+ */
 static bool tlshd_setsockopt(int sock, unsigned read, const void *info,
 			     socklen_t infolen)
 {
@@ -123,6 +160,15 @@ static bool tlshd_setsockopt(int sock, unsigned read, const void *info,
 }
 
 #if defined(TLS_CIPHER_AES_GCM_128)
+/**
+ * @brief Configure TLS session for AES-GCM-128 encryption
+ * @param[in]     session  An established TLS session
+ * @param[in]     sock     An open socket descriptor
+ * @param[in]     read     Read or write side
+ *
+ * @retval  true   The session was configured successfully
+ * @retval  false  The session was not configured
+ */
 static bool tlshd_set_aes_gcm128_info(gnutls_session_t session, int sock,
 				      unsigned read)
 {
@@ -162,6 +208,15 @@ static bool tlshd_set_aes_gcm128_info(gnutls_session_t session, int sock,
 #endif
 
 #if defined(TLS_CIPHER_AES_GCM_256)
+/**
+ * @brief Configure TLS session for AES-GCM-256 encryption
+ * @param[in]     session  An established TLS session
+ * @param[in]     sock     An open socket descriptor
+ * @param[in]     read     Read or write side
+ *
+ * @retval  true   The session was configured successfully
+ * @retval  false  The session was not configured
+ */
 static bool tlshd_set_aes_gcm256_info(gnutls_session_t session, int sock,
 				      unsigned read)
 {
@@ -201,6 +256,15 @@ static bool tlshd_set_aes_gcm256_info(gnutls_session_t session, int sock,
 #endif
 
 #if defined(TLS_CIPHER_AES_CCM_128)
+/**
+ * @brief Configure TLS session for AES-CCM-128 encryption
+ * @param[in]     session  An established TLS session
+ * @param[in]     sock     An open socket descriptor
+ * @param[in]     read     Read or write side
+ *
+ * @retval  true   The session was configured successfully
+ * @retval  false  The session was not configured
+ */
 static bool tlshd_set_aes_ccm128_info(gnutls_session_t session, int sock,
 				      unsigned read)
 {
@@ -240,6 +304,15 @@ static bool tlshd_set_aes_ccm128_info(gnutls_session_t session, int sock,
 #endif
 
 #if defined(TLS_CIPHER_CHACHA20_POLY1305)
+/**
+ * @brief Configure TLS session for ChaCha-Poly1305 encryption
+ * @param[in]     session  An established TLS session
+ * @param[in]     sock     An open socket descriptor
+ * @param[in]     read     Read or write side
+ *
+ * @retval  true   The session was configured successfully
+ * @retval  false  The session was not configured
+ */
 static bool tlshd_set_chacha20_poly1305_info(gnutls_session_t session, int sock,
 					     unsigned read)
 {
@@ -275,10 +348,10 @@ static bool tlshd_set_chacha20_poly1305_info(gnutls_session_t session, int sock,
 #endif
 
 /**
- * tlshd_initialize_ktls - Initialize socket for use by kTLS
- * @session: TLS session descriptor
+ * @brief Initialize a socket for use by kTLS
+ * @param[in]     session  TLS session descriptor
  *
- * Returns zero on success, or a positive errno value.
+ * @returns zero on success, or a positive errno value.
  */
 unsigned int tlshd_initialize_ktls(gnutls_session_t session)
 {
@@ -320,6 +393,13 @@ unsigned int tlshd_initialize_ktls(gnutls_session_t session)
 	return EIO;
 }
 
+/**
+ * @brief Concatenate a cipher name to a string
+ * @param[in]     pstring  NUL-terminated C string
+ * @param[in]     cipher   GnuTLS cipher number
+ *
+ * @retval A NUL-terminated C string; caller must free the string with free(3)
+ */
 static char *tlshd_cipher_string_emit(char *pstring, unsigned int cipher)
 {
 	switch (cipher) {
@@ -350,6 +430,13 @@ static gnutls_priority_t	tlshd_gnutls_priority_psk;
 static gnutls_priority_t	tlshd_gnutls_priority_psk_sha256;
 static gnutls_priority_t	tlshd_gnutls_priority_psk_sha384;
 
+/**
+ * @brief Initialize GnuTLS priority caches
+ * @param[in]     ciphers       Array of GnuTLS cipher numbers
+ * @param[in]     cipher_count  count of elements in "ciphers"
+ *
+ * @retval Zero on success; a negative errno if a failure occurred.
+ */
 static int tlshd_gnutls_priority_init_list(const unsigned int *ciphers,
 					   int cipher_count)
 {
@@ -357,7 +444,7 @@ static int tlshd_gnutls_priority_init_list(const unsigned int *ciphers,
 	const char *errpos;
 	int ret, i;
 
-	pstring = strdup("SECURE256:+SECURE128:-COMP-ALL");
+	pstring = strdup("-COMP-ALL");
 	if (!pstring)
 		return -ENOMEM;
 
@@ -425,7 +512,8 @@ static int tlshd_gnutls_priority_init_list(const unsigned int *ciphers,
 	}
 
 	tlshd_log_debug("x.509 priority string: %s\n", pstring);
-	ret = gnutls_priority_init(&tlshd_gnutls_priority_x509, pstring, &errpos);
+	ret = gnutls_priority_init2(&tlshd_gnutls_priority_x509, pstring, &errpos,
+				    GNUTLS_PRIORITY_INIT_DEF_APPEND);
 	if (ret != GNUTLS_E_SUCCESS) {
 		free(pstring_sha256);
 		free(pstring_sha384);
@@ -442,7 +530,8 @@ static int tlshd_gnutls_priority_init_list(const unsigned int *ciphers,
 	}
 
 	tlshd_log_debug("PSK priority string: %s\n", pstring);
-	ret = gnutls_priority_init(&tlshd_gnutls_priority_psk, pstring, &errpos);
+	ret = gnutls_priority_init2(&tlshd_gnutls_priority_psk, pstring, &errpos,
+				    GNUTLS_PRIORITY_INIT_DEF_APPEND);
 	if (ret != GNUTLS_E_SUCCESS) {
 		free(pstring_sha256);
 		free(pstring_sha384);
@@ -461,8 +550,9 @@ static int tlshd_gnutls_priority_init_list(const unsigned int *ciphers,
 	}
 
 	tlshd_log_debug("PSK SHA256 priority string: %s\n", pstring);
-	ret = gnutls_priority_init(&tlshd_gnutls_priority_psk_sha256,
-				   pstring, &errpos);
+	ret = gnutls_priority_init2(&tlshd_gnutls_priority_psk_sha256,
+				   pstring, &errpos,
+				   GNUTLS_PRIORITY_INIT_DEF_APPEND);
 	if (ret != GNUTLS_E_SUCCESS) {
 		free(pstring);
 		free(pstring_sha384);
@@ -482,8 +572,9 @@ static int tlshd_gnutls_priority_init_list(const unsigned int *ciphers,
 	}
 
 	tlshd_log_debug("PSK SHA384 priority string: %s\n", pstring);
-	ret = gnutls_priority_init(&tlshd_gnutls_priority_psk_sha384,
-				   pstring, &errpos);
+	ret = gnutls_priority_init2(&tlshd_gnutls_priority_psk_sha384,
+				   pstring, &errpos,
+				   GNUTLS_PRIORITY_INIT_DEF_APPEND);
 	if (ret != GNUTLS_E_SUCCESS) {
 		free(pstring);
 		gnutls_priority_deinit(tlshd_gnutls_priority_psk_sha256);
@@ -498,9 +589,9 @@ static int tlshd_gnutls_priority_init_list(const unsigned int *ciphers,
 }
 
 /**
- * tlshd_gnutls_priority_init - Initialize GnuTLS priority caches
+ * @brief Initialize GnuTLS priority caches
  *
- * Returns zero on success, or a negative errno value if a failure
+ * @returns zero on success, or a negative errno value if a failure
  * occurred.
  */
 int tlshd_gnutls_priority_init(void)
@@ -532,15 +623,15 @@ out:
 }
 
 /**
- * tlshd_gnutls_priority_set - Initialize priorities per-session
- * @session: session to initialize
- * @parms: handshake parameters
- * @psk_len: size of pre-shared key in bytes, or zero
+ * @brief Select GnuTLS priority cache to use for "session"
+ * @param[in]     session  Session to initialize
+ * @param[in]     parms    Handshake parameters
+ * @param[in]     psk_len  Size of pre-shared key in bytes, or zero
  *
- * Returns GNUTLS_E_SUCCESS on success, otherwise an error code.
+ * @returns GNUTLS_E_SUCCESS on success, otherwise an error code.
  */
 int tlshd_gnutls_priority_set(gnutls_session_t session,
-			      struct tlshd_handshake_parms *parms,
+			      const struct tlshd_handshake_parms *parms,
 			      unsigned int psk_len)
 {
 	gnutls_priority_t priority = tlshd_gnutls_priority_x509;
@@ -558,8 +649,7 @@ int tlshd_gnutls_priority_set(gnutls_session_t session,
 }
 
 /**
- * tlshd_gnutls_priority_deinit - Free GnuTLS priority caches
- *
+ * @brief Free GnuTLS priority caches
  */
 void tlshd_gnutls_priority_deinit(void)
 {
