@@ -1,6 +1,9 @@
+/**
+ * @file tlshd.h
+ * @brief Generic definitions and forward declarations for tlshd
+ */
+
 /*
- * Generic definitions and forward declarations for tlshd.
- *
  * ktls-utils is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; version 2.
@@ -18,6 +21,10 @@
 
 #include <linux/netlink.h>
 
+/**
+ * @def ARRAY_SIZE
+ * @brief Generate the number of elements in an array
+ */
 #define ARRAY_SIZE(a)		(sizeof(a) / sizeof((a)[0]))
 
 extern int tlshd_debug;
@@ -27,22 +34,32 @@ extern int tlshd_stderr;
 
 struct nl_sock;
 
+/**
+ * @struct tlshd_handshake_parms
+ * @brief Handshake parameters (global)
+ */
 struct tlshd_handshake_parms {
-	char		*peername;
-	char		*peeraddr;
-	int		sockfd;
-	int		ip_proto;
-	uint32_t	handshake_type;
-	unsigned int	timeout_ms;
-	uint32_t	auth_mode;
-	key_serial_t	keyring;
-	key_serial_t	x509_cert;
-	key_serial_t	x509_privkey;
-	GArray		*peerids;
-	GArray		*remote_peerids;
-	int		msg_status;
+	/*@{*/
+	char		*peername;	/**< Remote's DNS label */
+	char		*peeraddr;	/**< Remote's IP address */
+	int		sockfd;		/**< Socket on which to perform the handshake */
+	int		ip_proto;	/**< Transport protocol number */
+	uint32_t	handshake_type;	/**< Handshake interaction to perform */
+	unsigned int	timeout_ms;	/**< How long to wait for completion */
+	uint32_t	auth_mode;	/**< x.509, PSK, etc. */
+	key_serial_t	keyring;	/**< Keyring containing auth material */
+	key_serial_t	x509_cert;	/**< Key serial of our x.509 cert */
+	key_serial_t	x509_privkey;	/**< Key serial of our x.509 private key */
+	GArray		*peerids;	/**< Peer identities to present to servers */
+	GArray		*remote_peerids; /**< Peer identities presented by clients */
 
-	unsigned int	session_status;
+	unsigned int	session_status;	/**< Handshake completion status */
+	/*@}*/
+};
+
+enum peer_type {
+	PEER_TYPE_CLIENT,
+	PEER_TYPE_SERVER,
 };
 
 /* client.c */
@@ -50,18 +67,16 @@ extern void tlshd_tls13_clienthello_handshake(struct tlshd_handshake_parms *parm
 extern void tlshd_quic_clienthello_handshake(struct tlshd_handshake_parms *parms);
 
 /* config.c */
-bool tlshd_config_init(const gchar *pathname);
+bool tlshd_config_init(const gchar *pathname, bool legacy);
 void tlshd_config_shutdown(void);
-bool tlshd_config_get_client_truststore(char **bundle);
-bool tlshd_config_get_client_crl(char **result);
-bool tlshd_config_get_client_certs(gnutls_pcert_st *certs,
-				   unsigned int *certs_len);
-bool tlshd_config_get_client_privkey(gnutls_privkey_t *privkey);
-bool tlshd_config_get_server_truststore(char **bundle);
-bool tlshd_config_get_server_crl(char **result);
-bool tlshd_config_get_server_certs(gnutls_pcert_st *certs,
-				   unsigned int *certs_len);
-bool tlshd_config_get_server_privkey(gnutls_privkey_t *privkey);
+bool tlshd_config_get_truststore(int peer_type, char **bundle);
+bool tlshd_config_get_crl(int peer_type, char **result);
+bool tlshd_config_get_certs(int peer_type, gnutls_pcert_st *certs,
+			    unsigned int *pq_certs_len,
+			    unsigned int *certs_len,
+			    gnutls_pk_algorithm_t *pkalg);
+bool tlshd_config_get_privkey(int peer_type, gnutls_privkey_t *pq_privkey,
+			      gnutls_privkey_t *privkey);
 
 /* handshake.c */
 extern void tlshd_start_tls_handshake(gnutls_session_t session,
@@ -85,7 +100,7 @@ extern int tlshd_keyring_link_session(const char *keyring);
 extern unsigned int tlshd_initialize_ktls(gnutls_session_t session);
 extern int tlshd_gnutls_priority_init(void);
 extern int tlshd_gnutls_priority_set(gnutls_session_t session,
-				     struct tlshd_handshake_parms *parms,
+				     const struct tlshd_handshake_parms *parms,
 				     unsigned int psk_len);
 extern void tlshd_gnutls_priority_deinit(void);
 
@@ -120,11 +135,22 @@ extern void tlshd_tls13_serverhello_handshake(struct tlshd_handshake_parms *parm
 extern void tlshd_quic_serverhello_handshake(struct tlshd_handshake_parms *parms);
 
 #ifdef HAVE_GNUTLS_QUIC
-#include <linux/quic.h>
+#include "quic.h"
+
+#ifndef SOL_QUIC
+#define SOL_QUIC	288
+#endif
+#ifndef IPPROTO_QUIC
+#define IPPROTO_QUIC	261
+#endif
 
 #define TLSHD_QUIC_MAX_DATA_LEN		4096
 #define TLSHD_QUIC_MAX_ALPNS_LEN	128
 
+/**
+ * @struct tlshd_quic_msg
+ * @brief QUIC message format
+ */
 struct tlshd_quic_msg {
 	struct tlshd_quic_msg *next;
 	uint8_t data[TLSHD_QUIC_MAX_DATA_LEN];
@@ -132,6 +158,10 @@ struct tlshd_quic_msg {
 	uint8_t level;
 };
 
+/**
+ * @struct tlshd_quic_conn
+ * @brief QUIC connection object
+ */
 struct tlshd_quic_conn {
 	struct tlshd_handshake_parms *parms;
 	char alpns[TLSHD_QUIC_MAX_ALPNS_LEN];
@@ -152,17 +182,39 @@ struct tlshd_quic_conn {
 	struct tlshd_quic_msg recv_msg;
 };
 
-/* quic.c */
 extern int tlshd_quic_conn_create(struct tlshd_quic_conn **conn_p,
 				  struct tlshd_handshake_parms *parms);
 extern void tlshd_quic_conn_destroy(struct tlshd_quic_conn *conn);
 extern void tlshd_quic_start_handshake(struct tlshd_quic_conn *conn);
+
 #endif
 
-#define TLS_DEFAULT_PRIORITIES	(NULL)
+/**
+ * @def TLS_DEFAULT_PSK_TYPE
+ * @brief Default type of pre-shared key
+ */
 #define TLS_DEFAULT_PSK_TYPE	"psk"
+
+/**
+ * @def TLS_NO_PEERID
+ * @brief No peer ID provided via keyring
+ */
 #define TLS_NO_PEERID		(0)
+
+/**
+ * @def TLS_NO_CERT
+ * @brief No certificate provided via keyring
+ */
 #define TLS_NO_CERT		(0)
+
+/**
+ * @def TLS_NO_PRIVKEY
+ * @brief No private key provided via keyring
+ */
 #define TLS_NO_PRIVKEY		(0)
-/* Max number of (chained) certs to load */
+
+/**
+ * @def TLSHD_MAX_CERTS
+ * @brief Maximum number of (chained) certs to load
+ */
 #define TLSHD_MAX_CERTS		10
